@@ -30,6 +30,14 @@
 #' The \link{multi_cancel} function can be used to cancel a pending request.
 #' It has no effect if the request was already completed or canceled.
 #'
+#' The \link{multi_fdset} function returns the file descriptors curl is
+#' polling currently, and also a timeout parameter, the number of
+#' milliseconds an application should wait (at most) before proceeding. It
+#' is equivalent to the \code{curl_multi_fdset} and
+#' \code{curl_multi_timeout} calls. It is handy for applications that is
+#' expecting input (or writing output) through both curl, and other file
+#' descriptors.
+#'
 #' @name multi
 #' @rdname multi
 #' @useDynLib curl R_multi_add
@@ -38,24 +46,65 @@
 #' response data in same structure as \link{curl_fetch_memory}.
 #' @param fail callback function called on failed request. Argument contains
 #' error message.
+#' @param data callback function or open connection object for receiving data.
+#' If \code{NULL} the entire response content gets buffered and is returned
+#' in the \code{done} callback.
 #' @param pool a multi handle created by \link{new_pool}. Default uses a global pool.
 #' @export
-#' @examples h1 <- new_handle(url = "https://eu.httpbin.org/delay/3")
+#' @examples
+#' results <- list()
+#' success <- function(x){
+#'   results <<- append(results, list(x))
+#' }
+#' failure <- function(str){
+#'   cat(paste("Failed request:", str), file = stderr())
+#' }
+#' # This handle will take longest (3sec)
+#' h1 <- new_handle(url = "https://eu.httpbin.org/delay/3")
+#' multi_add(h1, done = success, fail = failure)
+#'
+#' # This handle writes data to a file
+#' con <- file("output.txt", open = "wb")
 #' h2 <- new_handle(url = "https://eu.httpbin.org/post", postfields = "bla bla")
+#' multi_add(h2, done = success, fail = failure, data = con)
+#'
+#' # This handle raises an error
 #' h3 <- new_handle(url = "https://urldoesnotexist.xyz")
-#' multi_add(h1, done = print, fail = print)
-#' multi_add(h2, done = print, fail = print)
-#' multi_add(h3, done = print, fail = print)
+#' multi_add(h3, done = success, fail = failure)
+#'
+#' # Actually perform the requests
 #' multi_run(timeout = 2)
 #' multi_run()
-multi_add <- function(handle, done = NULL, fail = NULL, pool = NULL){
+#'
+#' # Check the file
+#' close(con)
+#' readLines("output.txt")
+#' unlink("output.txt")
+multi_add <- function(handle, done = NULL, fail = NULL, data = NULL, pool = NULL){
   if(is.null(pool))
     pool <- multi_default()
+  if(inherits(data, "connection")){
+    con <- data
+    if(!isOpen(con))
+      stop("Connection for 'data' argument is not open")
+    data <- if(identical(summary(data)$text, "text")){
+      function(x){
+        cat(rawToChar(x), file = con)
+        flush(con)
+      }
+    } else {
+      function(x){
+        writeBin(x, con = con)
+        flush(con)
+      }
+    }
+  }
   stopifnot(inherits(handle, "curl_handle"))
   stopifnot(inherits(pool, "curl_multi"))
   stopifnot(is.null(done) || is.function(done))
   stopifnot(is.null(fail) || is.function(fail))
-  .Call(R_multi_add, handle, done, fail, pool)
+  stopifnot(is.null(data) || is.function(data))
+  .Call(R_multi_add, handle, done, fail, data, pool)
 }
 
 #' @param timeout max time in seconds to wait for results. Use \code{0} to poll for results without
@@ -131,3 +180,14 @@ print.curl_multi <- function(x, ...){
   len <- length(multi_list(x))
   cat(sprintf("<curl multi-pool> (%d pending requests)\n", len))
 }
+
+#' @export
+#' @useDynLib curl R_multi_fdset
+#' @rdname multi
+
+multi_fdset <- function(pool = NULL){
+  if(is.null(pool))
+    pool <- multi_default()
+  stopifnot(inherits(pool, "curl_multi"))
+  .Call(R_multi_fdset, pool)
+}
